--
-- Mogli Hud 
--
-- @author  Mogli aka biedens
-- @date  17.12.2013
--
 
Mogli = {};

------------------------------------------------------------------------
-- some local helper functions
------------------------------------------------------------------------
function Mogli.bool2int(boolean)
	if boolean then
		return 1;
	end;
	return 0;
end;

function Mogli.printCallstack()
	local i = 2;
	local info;
	print("------------------------------------------------------------------------");
	while i <= 10 do
		info = debug.getinfo(i);
		if info == nil then break end
		print(string.format("%i: %s (%i): %s", i, info.short_src, Utils.getNoNil(info.currentline,0), Utils.getNoNil(info.name,"<???>")));
		i = i + 1;
	end
	if info ~= nil and info.name ~= nil and info.currentline ~= nil then
		print("...");
	end
	print("------------------------------------------------------------------------");
end

function Mogli.getText(id)
	if id == nil then
		return "nil";
	end;
	
	if g_i18n:hasText( id ) then
	  return g_i18n:getText( id )
	end
	
	return id
end;

function Mogli.getXmlBool(xmlFile, key, default)
	local l = getXMLInt(xmlFile, key);
	if l~= nil then
		return (l == 1);
	end;
	return default;
end;

function Mogli.getXmlFloat(xmlFile, key, default)
	local f = getXMLFloat(xmlFile, key);
	if f ~= nil then
		return f;
	end;
	return default;
end;

function Mogli.getXmlInt(xmlFile, key, default)
	local f = getXMLInt(xmlFile, key);
	if f ~= nil then
		return f;
	end;
	return default;
end;

------------------------------------------------------------------------
-- init
------------------------------------------------------------------------
function Mogli:init( directory, hudName, hudBackground, onTextID, offTextID, showHudKey, x,y, nx, ny, cbOnClick, w, h, buttonBackground )
	self.mogliDirectory = directory;

	if self.isClient and ( self.mouseEventRedefined == nil or not self.mouseEventRedefined ) then
		self.mouseEventRedefined = true
		self.mouseEvent          = Utils.overwrittenFunction( self.mouseEvent, Mogli.newMouseEvent );			
	end
		
	if w == nil or w <= 0.01 then
		self.mogliHudBtnWidth  = 0.03;     
	else
		self.mogliHudBtnWidth  = w;     
	end
	if h == nil or h <= 0.01 then
		self.mogliHudBtnHeight  = 0.04;     
	else
		self.mogliHudBtnHeight = h;     
	end
	
	self.mogliHudWidth     = nx * ( self.mogliHudBtnWidth  + 0.01) + 0.01
	self.mogliHudHeight    = ( ny + 2 ) * ( self.mogliHudBtnHeight + 0.01 ) - 0.01
	self.mogliHudPosX      = x;   
	self.mogliHudPosY      = y;  
	self.mogliHudBtnPosX   = x + 0.01 ;     
	self.mogliHudBtnPosY   = y + ny * ( self.mogliHudBtnHeight + 0.01 );
	self.mogliHudTextPosX  = self.mogliHudBtnPosX;--0.024;     
	self.mogliHudTextPosY  = y + 0.01;
	self.mogliHudTitlePosY = y + ( ny + 1 ) * ( self.mogliHudBtnHeight + 0.01 );
	self.mogliHudOverlay   = MogliButton:new(hudName, "dataS2/menu/blank.png", self.mogliHudPosX, self.mogliHudPosY, self.mogliHudWidth, self.mogliHudHeight);
	self.mogliHudOverlay:setColor(0,0,0,hudBackground)
	
	self.mogliGuiActive    = false;
	self.mogliGuiShowKey   = showHudKey;
	self.mogliGuiOnTextID  = onTextID;
	self.mogliGuiOffTextID = offTextID;	
	self.mogliCBOnClick    = cbOnClick;
	self.mogliStatus       = 0
	self.mogliTitle        = ""
	self.mogliButtonBgr    = buttonBackground
	
	Mogli.addCloseButton( self, nx, ny )
end

------------------------------------------------------------------------
-- addButton
------------------------------------------------------------------------
function Mogli:addButton(imgEnabled, imgDisabled, cbOnClick, cbVisible, nx, ny, textEnabled, textDisabled, textCallback, imgCallback)
	local x = self.mogliHudBtnPosX + (nx-1)*(self.mogliHudBtnWidth+0.01);
	local y = self.mogliHudBtnPosY - (ny-1)*(self.mogliHudBtnHeight+0.01);
	local background = nil
	local overlay    = nil
	local overlay2   = nil
	if self.mogliButtonBgr ~= nil then
		background = MogliButton:new(nil, "dataS2/menu/blank.png", x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
		self.mogliHudOverlay:setColor(0,0,0,self.mogliButtonBgr)
	end
	if imgEnabled ~= nil then
		overlay = MogliButton:new(nil, Utils.getFilename(imgEnabled, self.mogliDirectory), x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
	elseif self.mogliButtonBgr == nil then
	--overlay = MogliButton:new(nil, "dataS2/menu/blank.png", x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
		overlay = MogliButton:new(nil, Utils.getFilename("dds/empty.dds", self.mogliDirectory), x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
	end
	if imgDisabled ~= nil then
		overlay2 = MogliButton:new(nil, Utils.getFilename(imgDisabled, self.mogliDirectory), x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
	elseif self.mogliButtonBgr == nil then
	--overlay2 = MogliButton:new(nil, "dataS2/menu/blank.png", x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
		overlay2 = MogliButton:new(nil, Utils.getFilename("dds/empty.dds", self.mogliDirectory), x,y,self.mogliHudBtnWidth,self.mogliHudBtnHeight);
	end
	
	local button = {enabled=true, ovEnabled=overlay, ovDisabled=overlay2, ovBackground=background, onClick=cbOnClick, onVisible=cbVisible, twoState=(imgDisabled ~= nil), rect={x,y,x+self.mogliHudBtnWidth,y+self.mogliHudBtnHeight}, text1 = textEnabled, text2 = textDisabled, textcb = textCallback, onRender = imgCallback };
	button.overlays = {}
	if imgDisabled ~= nil then
		button.overlays[imgDisabled] = overlay2
	end
	if imgEnabled ~= nil then
		button.overlays[imgEnabled] = overlay
	end
	if self.mogliButtons == nil then self.mogliButtons = {}; end
	table.insert(self.mogliButtons, button);
	return button;
end;

------------------------------------------------------------------------
-- onClose
------------------------------------------------------------------------
function Mogli:onClose()
	Mogli.showGui(self,false)
end

------------------------------------------------------------------------
-- addCloseButton
------------------------------------------------------------------------
function Mogli:addCloseButton(nx, ny)
	local x = self.mogliHudBtnPosX + (nx-1)*(self.mogliHudBtnWidth+0.01) + 0.5*self.mogliHudBtnWidth;
	local y = self.mogliHudBtnPosY + self.mogliHudBtnHeight+0.01;
	local overlay = MogliButton:new(nil, Utils.getFilename("dds/close.dds", self.mogliDirectory), x,y,0.5*self.mogliHudBtnWidth,0.5*self.mogliHudBtnHeight);
	local button = {enabled=true, ovEnabled=overlay, ovDisabled=nil, ovBackground=nil, onClick=Mogli.onClose, onVisible=nil, twoState=false, rect={x,y,x+0.5*self.mogliHudBtnWidth,y+0.5*self.mogliHudBtnHeight}, text1 = nil, text2 = nil, textcb = nil, onRender = nil };
	button.overlays = {}
	button.overlays["dds/close.dds"] = overlay
	if self.mogliButtons == nil then self.mogliButtons = {}; end
	table.insert(self.mogliButtons, button);
	return button;
end;

------------------------------------------------------------------------
-- setStatus
------------------------------------------------------------------------
function Mogli:setStatus(status)
	if status == nil or status == 0 then
		self.mogliStatus = 0
	else
		self.mogliStatus = status
	end
end

------------------------------------------------------------------------
-- setTitle
------------------------------------------------------------------------
function Mogli:setTitle(title)
	if title == nil then
		self.mogliTitle = ""
	else
		self.mogliTitle = Mogli.getText( title )
	end
end

------------------------------------------------------------------------
-- newMouseEvent
------------------------------------------------------------------------
function Mogli:newMouseEvent(superFunc, posX, posY, isDown, isUp, button)
	if self.mogliGuiActive then
		local x = InputBinding.mouseMovementX;
		local y = InputBinding.mouseMovementY;
		InputBinding.mouseMovementX = 0;
		InputBinding.mouseMovementY = 0;
		superFunc(self,posX, posY, isDown, isUp, button);
		InputBinding.mouseMovementX = x;
		InputBinding.mouseMovementY = y;
	else
		superFunc(self,posX, posY, isDown, isUp, button);
	end;
end;

------------------------------------------------------------------------
-- mouseEvent
------------------------------------------------------------------------
function Mogli:mouseEvent(posX, posY, isDown, isUp, button)

	self.mogliTooltip = nil;
	local textID, textCB;
	if self.mogliGuiActive then
		for _,overlayButton in pairs(self.mogliButtons) do
			if overlayButton.rect[1] <= posX and posX <= overlayButton.rect[3] and overlayButton.rect[2] <= posY and posY <= overlayButton.rect[4] then
				if overlayButton.onClick ~= nil and isDown and button == 1 then
					if overlayButton.enabled then
						if overlayButton.twoState ~= nil then
							overlayButton.onClick(self, true);
						else
							overlayButton.onClick(self);
						end;
						self.mogliCBOnClick(self);
					elseif overlayButton.twoState then
						overlayButton.onClick(self, false);
						self.mogliCBOnClick(self);
					end;
				end
				if  overlayButton.text1 ~= nil 
						and ( overlayButton.enabled 
							or not overlayButton.twoState 
							or overlayButton.text2 == nil ) then
					textID = overlayButton.text1;
				elseif overlayButton.text2 ~= nil then
					textID = overlayButton.text2;
				end;
				textCB = overlayButton.textcb;
				break;				
			end;
		end;
	end;
	
	if textID ~= nil then
		self.mogliTooltip = Mogli.getText( textID );
		if textCB ~= nil then 
			self.mogliTooltip = textCB(self,self.mogliTooltip) 
		end
	end
end

------------------------------------------------------------------------
-- renderButtons
------------------------------------------------------------------------
function Mogli:renderButtons()
  for _,button in pairs(self.mogliButtons) do
    if button.onVisible ~= nil then
			button.enabled = button.onVisible(self);
		end;
		local img = nil
		if button.onRender ~= nil then
			img = button.onRender(self)
		end
		if button.ovBackground ~= nil then
			button.ovBackground:render();
		end
		if img ~= nil and img ~= "" then
			if button.overlays == nil then
				button.overlays = {}
			end
			local ov
			if button.overlays[img] == nil then
				ov = MogliButton:new(nil, Utils.getFilename(img, self.mogliDirectory), button.rect[1],button.rect[2],self.mogliHudBtnWidth,self.mogliHudBtnHeight);
				button.overlays[img] = ov
			else
				ov = button.overlays[img]
			end
			ov:render()
		elseif button.enabled then
			if button.ovEnabled ~= nil then
				button.ovEnabled:render();
			end;
		elseif button.ovDisabled ~= nil then
			button.ovDisabled:render();
		end;
  end;	
end;

------------------------------------------------------------------------
-- mouse event callbacks
------------------------------------------------------------------------
function Mogli:showGui(on)	
	local old = self.mogliGuiActive
	if self.isClient then
		self.mogliGuiActive = on;
	else
		self.mogliGuiActive = false
	end
	if old ~= self.mogliGuiActive then
		g_mouseControlsHelp.active = not on;
		InputBinding.setShowMouseCursor(on);		
	end
end;

------------------------------------------------------------------------
-- draw
------------------------------------------------------------------------
function Mogli:draw(hideKey)
	if self.isClient then
		if self.mogliGuiActive then
			if hideKey == nil or not hideKey then
				g_currentMission:addHelpButtonText(Mogli.getText(self.mogliGuiOffTextID), self.mogliGuiShowKey);
			end
			g_mouseControlsHelp.active = false;
			InputBinding.setShowMouseCursor(true);		
			setTextAlignment(RenderText.ALIGN_LEFT);
			
			setTextBold(true);		
			if self.mogliStatus == 0 then
				setTextColor(1,1,1,1);
			elseif self.mogliStatus == 1 then
				setTextColor(0,1,0,1);
			elseif self.mogliStatus == 2 then
				setTextColor(1,1,0,1);
			else
				setTextColor(1,0.5,0,1);
			end
			
			renderText(self.mogliHudTextPosX, self.mogliHudTitlePosY, 0.021,self.mogliTitle);		
			
			setTextBold(false);		
			setTextColor(1,1,1,1);
						
			self.mogliHudOverlay:render();
			if     self.mogliTooltip           ~= nil and self.mogliTooltip           ~= "" then
				renderText(self.mogliHudTextPosX, self.mogliHudTextPosY, 0.021,self.mogliTooltip);
			elseif self.mogliInfoText ~= nil and self.mogliInfoText ~= "" then
				renderText(self.mogliHudTextPosX, self.mogliHudTextPosY, 0.021,self.mogliInfoText);
			end
			Mogli.renderButtons(self);
		elseif hideKey == nil or not hideKey then
			g_currentMission:addHelpButtonText(Mogli.getText(self.mogliGuiOnTextID), self.mogliGuiShowKey);
		end
	end
end

------------------------------------------------------------------------
-- delete
------------------------------------------------------------------------
function Mogli:delete()
end;

------------------------------------------------------------------------
-- onLeave
------------------------------------------------------------------------
function Mogli:onLeave()
	if self.isClient and self.mogliGuiActive then
		g_mouseControlsHelp.active = true;
		InputBinding.setShowMouseCursor(false);		
	end
end;

------------------------------------------------------------------------
-- onEnter
------------------------------------------------------------------------
function Mogli:onEnter()
	Mogli.showGui(self, self.mogliGuiActive);
	if self.isClient and self.mogliGuiActive then
		g_mouseControlsHelp.active = false
		InputBinding.setShowMouseCursor(true);		
	end
end;

------------------------------------------------------------------------
-- MogliButton
------------------------------------------------------------------------
MogliButton = {}
local MogliButton_mt = Class(MogliButton)
MogliButton.overlayIds = {}

function MogliButton.getOverlayId( imageFileName )
	if MogliButton.overlayIds[imageFileName] == nil then
		MogliButton.overlayIds[imageFileName] = createImageOverlay(imageFileName)
	end
	return MogliButton.overlayIds[imageFileName]
end

function MogliButton.onDeleteMap()
	for name,id in pairs( MogliButton.overlayIds ) do
	--print("Removing image file "..name.." from memory...")
		pcall( delete, id )
	end
	MogliButton.overlayIds = {}
end

function MogliButton:new(name, overlayFilename, x, y, width, height)
  local overlayId = 0
  if overlayFilename ~= nil then
    overlayId = MogliButton.getOverlayId( overlayFilename )
  end
  return setmetatable({
    overlayId = overlayId,
    x = x,
    y = y,
    width = width,
    height = height,
    visible = true,
    invertX = false,
    rotation = 0,
    rotationCenterX = 0,
    rotationCenterY = 0,
    r = 1,
    g = 1,
    b = 1,
    a = 1
  }, MogliButton_mt)
end
function MogliButton:delete()
  if self.overlayId ~= 0 then
    delete(self.overlayId)
  end
end
function MogliButton:setColor(r, g, b, a)
  self.r, self.g, self.b, self.a = r, g, b, a
end
function MogliButton:setPosition(x, y)
  self.x = x
  self.y = y
end
function MogliButton:setDimension(width, height)
  self.width = width
  self.height = height
end
function MogliButton:render()
  if self.visible and self.overlayId ~= 0 then
    setOverlayColor(self.overlayId, self.r, self.g, self.b, self.a)
    renderOverlay(self.overlayId, self.x, self.y, self.width, self.height)
  end
end

